/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_DMSERVICE_DMSERVICE_SKELETON_H
#define JAC_AP_DMSERVICE_DMSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/dmservice/dmservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace dmservice {
namespace skeleton {
namespace events
{
    using Decision4SMEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::Decision4SM>;
    using ObjectTrimEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::ObjectTrim>;
    using Decision4LongEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::Decision4Long>;
    using Decision4LatEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::Decision4Lat>;
    using DM2LongRefLineEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::DM2LongRefLine>;
    static constexpr ara::com::internal::EntityId DMServiceDecision4SMEventId = 44931U; //Decision4SMEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceObjectTrimEventId = 57390U; //ObjectTrimEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceDecision4LongEventId = 22274U; //Decision4LongEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceDecision4LatEventId = 4690U; //Decision4LatEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceDM2LongRefLineEventId = 19701U; //DM2LongRefLineEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class DMServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultDecision4SMEvent = Decision4SMEvent.Initialize();
        ThrowError(resultDecision4SMEvent);
        const ara::core::Result<void> resultObjectTrimEvent = ObjectTrimEvent.Initialize();
        ThrowError(resultObjectTrimEvent);
        const ara::core::Result<void> resultDecision4LongEvent = Decision4LongEvent.Initialize();
        ThrowError(resultDecision4LongEvent);
        const ara::core::Result<void> resultDecision4LatEvent = Decision4LatEvent.Initialize();
        ThrowError(resultDecision4LatEvent);
        const ara::core::Result<void> resultDM2LongRefLineEvent = DM2LongRefLineEvent.Initialize();
        ThrowError(resultDM2LongRefLineEvent);
    }

    DMServiceSkeleton& operator=(const DMServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId, mode)),
              Decision4SMEvent(ptr->GetSkeleton(), events::DMServiceDecision4SMEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
              ObjectTrimEvent(ptr->GetSkeleton(), events::DMServiceObjectTrimEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
              Decision4LongEvent(ptr->GetSkeleton(), events::DMServiceDecision4LongEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
              Decision4LatEvent(ptr->GetSkeleton(), events::DMServiceDecision4LatEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
              DM2LongRefLineEvent(ptr->GetSkeleton(), events::DMServiceDM2LongRefLineEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec, mode)),
              Decision4SMEvent(ptr->GetSkeleton(), events::DMServiceDecision4SMEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
              ObjectTrimEvent(ptr->GetSkeleton(), events::DMServiceObjectTrimEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
              Decision4LongEvent(ptr->GetSkeleton(), events::DMServiceDecision4LongEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
              Decision4LatEvent(ptr->GetSkeleton(), events::DMServiceDecision4LatEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
              DM2LongRefLineEvent(ptr->GetSkeleton(), events::DMServiceDM2LongRefLineEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer, mode)),
              Decision4SMEvent(ptr->GetSkeleton(), events::DMServiceDecision4SMEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
              ObjectTrimEvent(ptr->GetSkeleton(), events::DMServiceObjectTrimEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
              Decision4LongEvent(ptr->GetSkeleton(), events::DMServiceDecision4LongEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
              Decision4LatEvent(ptr->GetSkeleton(), events::DMServiceDecision4LatEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
              DM2LongRefLineEvent(ptr->GetSkeleton(), events::DMServiceDM2LongRefLineEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              Decision4SMEvent(std::move(other.Decision4SMEvent)),
              ObjectTrimEvent(std::move(other.ObjectTrimEvent)),
              Decision4LongEvent(std::move(other.Decision4LongEvent)),
              Decision4LatEvent(std::move(other.Decision4LatEvent)),
              DM2LongRefLineEvent(std::move(other.DM2LongRefLineEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                Decision4SMEvent = std::move(other.Decision4SMEvent);
                ObjectTrimEvent = std::move(other.ObjectTrimEvent);
                Decision4LongEvent = std::move(other.Decision4LongEvent);
                Decision4LatEvent = std::move(other.Decision4LatEvent);
                DM2LongRefLineEvent = std::move(other.DM2LongRefLineEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::Decision4SMEvent GetDecision4SMEvent()
        {
            return std::move(Decision4SMEvent);
        }
        events::ObjectTrimEvent GetObjectTrimEvent()
        {
            return std::move(ObjectTrimEvent);
        }
        events::Decision4LongEvent GetDecision4LongEvent()
        {
            return std::move(Decision4LongEvent);
        }
        events::Decision4LatEvent GetDecision4LatEvent()
        {
            return std::move(Decision4LatEvent);
        }
        events::DM2LongRefLineEvent GetDM2LongRefLineEvent()
        {
            return std::move(DM2LongRefLineEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = Decision4SMEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = ObjectTrimEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = Decision4LongEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = Decision4LatEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = DM2LongRefLineEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::Decision4SMEvent Decision4SMEvent;
        events::ObjectTrimEvent ObjectTrimEvent;
        events::Decision4LongEvent Decision4LongEvent;
        events::Decision4LatEvent Decision4LatEvent;
        events::DM2LongRefLineEvent DM2LongRefLineEvent;
    };
    explicit DMServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId, mode)),
          Decision4SMEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4SMEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
          ObjectTrimEvent(skeletonAdapter->GetSkeleton(), events::DMServiceObjectTrimEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
          Decision4LongEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4LongEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
          Decision4LatEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4LatEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId),
          DM2LongRefLineEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDM2LongRefLineEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit DMServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec, mode)),
          Decision4SMEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4SMEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
          ObjectTrimEvent(skeletonAdapter->GetSkeleton(), events::DMServiceObjectTrimEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
          Decision4LongEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4LongEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
          Decision4LatEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4LatEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec),
          DM2LongRefLineEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDM2LongRefLineEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit DMServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer, mode)),
          Decision4SMEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4SMEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
          ObjectTrimEvent(skeletonAdapter->GetSkeleton(), events::DMServiceObjectTrimEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
          Decision4LongEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4LongEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
          Decision4LatEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDecision4LatEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer),
          DM2LongRefLineEvent(skeletonAdapter->GetSkeleton(), events::DMServiceDM2LongRefLineEventId, ::jac::ap::dmservice::DMService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    DMServiceSkeleton(const DMServiceSkeleton&) = delete;

    DMServiceSkeleton(DMServiceSkeleton&&) = default;
    DMServiceSkeleton& operator=(DMServiceSkeleton&&) = default;
    DMServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          Decision4SMEvent(token.GetDecision4SMEvent()),
          ObjectTrimEvent(token.GetObjectTrimEvent()),
          Decision4LongEvent(token.GetDecision4LongEvent()),
          Decision4LatEvent(token.GetDecision4LatEvent()),
          DM2LongRefLineEvent(token.GetDM2LongRefLineEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~DMServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&DMServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::Decision4SMEvent Decision4SMEvent;
    events::ObjectTrimEvent ObjectTrimEvent;
    events::Decision4LongEvent Decision4LongEvent;
    events::Decision4LatEvent Decision4LatEvent;
    events::DM2LongRefLineEvent DM2LongRefLineEvent;
};
} // namespace skeleton
} // namespace dmservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_DMSERVICE_DMSERVICE_SKELETON_H
